<?php

namespace Drupal\config_enforce_devel\Form;

use Drupal\config_enforce\ConfigEnforce;
use Drupal\config_enforce\Form\AbstractConfigEnforceForm;
use Drupal\config_enforce_devel\ConfigEnforceDevel;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Markup;

/**
 * Defines a settings form for the Config Enforce Devel module.
 */
class ConfigEnforceDevelForm extends AbstractConfigEnforceForm {

  const FORM_ID = 'config_enforce_devel';

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    parent::buildForm($form, $form_state);

    if (empty($this->getFormConfigs())) {
      return $this->unsupportedWarning();
    }

    foreach ($this->getFormConfigs() as $config) {
      $enforced = ConfigEnforce::isEnforced($config);
      $this->form['config_enforce'][$config]['config_enforce_enabled'] = [
        '#type' => 'checkbox',
        '#title' => 'Enforce config',
        '#description' => "Enable enforcement of the config from this form.",
        '#default_value' => $enforced,
      ];

      $states = [
        'invisible' => [
          ':input[name="config_enforce[' . $config . '][config_enforce_enabled]"]' => ['checked' => FALSE],
        ],
      ];

      $module = ConfigEnforce::getModule($config);
      $module_title = $this->t('Module');
      if ($enforced) {
        $module_title = $this->t('Module (%current)', ['%current' => $module]);
      }
      $this->form['config_enforce'][$config]['config_enforce_module'] = [
        '#type' => 'select',
        '#title' => $module_title,
        '#description' => "The module to which this form's config should be written.",
        '#options' => $this->getAvailableModules(),
        '#default_value' => $module,
        '#states' => $states,
      ];

      $directory = ConfigEnforce::getDirectory($config);
      $directory_title = $this->t('Directory');
      if ($enforced) {
        $module_title = $this->t('Directory (%current)', ['%current' => $directory]);
      }
      $this->form['config_enforce'][$config]['config_enforce_directory'] = [
        '#type' => 'select',
        '#title' => $directory_title,
        '#description' => "The config directory to which this form's config should be written.",
        '#options' => $this->getAvailableDirectories(),
        '#default_value' => $directory,
        '#states' => $states,
      ];

      if ($enforced) {
        $this->form['config_enforce'][$config]['config_enforce_path']['#states'] = $states;
      }

      $levels = ConfigEnforce::getLevels();
      $level = ConfigEnforce::getLevel($config);
      $level_title = $this->t('Enforcement level');
      if ($enforced) {
        $level_title = $this->t('Enforcement level (%current)', ['%current' => $levels[$level]]);
      }
      $this->form['config_enforce'][$config]['config_enforce_level'] = [
        '#type' => 'select',
        '#title' => $level_title,
        '#description' => "The level of enforcement to apply to this form's config.",
        '#options' => $levels,
        '#default_value' => $level,
        '#states' => $states,
      ];
    }

    return parent::buildBaseForm($this->form, $this->formState);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);
    $this->saveConfig();
  }

  /**
   * Save our config.
   */
  protected function saveConfig() {
    $config_enforce = new ConfigEnforceDevel();
    foreach ($this->getFormConfigs() as $config) {
      $config_enforce->setConfig($config);

      if (!$this->getEnforced($config)) {
        $config_enforce->clear();
        continue;
      }

      $config_enforce
        ->setModule($this->getModule($config))
        ->setDirectory($this->getDirectory($config))
        ->setLevel($this->getLevel($config))
        ->save();
    }
  }

  /**
   * Return a warning that the config form is unsupported.
   */
  protected function unsupportedWarning() {
    $warning = $this->t(
      'This configuration form (@form_id) is not yet supported by Config Enforce.',
      ['@form_id' => $this->getContext('form_id')]
    );
    $this->form['unsupported'] = [
      '#type' => 'markup',
      '#markup' => Markup::create("
        <div class='messages messages--warning'>
          {$warning}
        </div>
      "),
    ];
    $this->form['#attached']['library'][] = 'classy/messages';
    return $this->form;
  }

  /**
   * Return a list of available modules to target for writing config.
   */
  protected function getAvailableModules() {
    return $this->config('config_enforce_devel.settings')->get('available_modules');
  }

  /**
   * Return a list of directories for writing config.
   */
  protected function getAvailableDirectories() {
    // @TODO Look this up in the storage service or make this a hook.
    return [
      'install' => 'Install',
      'optional' => 'Optional',
    ];
  }

  /**
   * Return array of config enforce values from the form.
   */
  protected function getConfigEnforceFromForm() {
    return $this->formState->getValue('config_enforce');
  }

  /**
   * Return the new target module from the form state.
   */
  protected function getModule($config) {
    return $this->getConfigEnforceFromForm()[$config]['config_enforce_module'];
  }

  /**
   * Return the new config directory from the form state.
   */
  protected function getDirectory($config) {
    return $this->getConfigEnforceFromForm()[$config]['config_enforce_directory'];
  }

  /**
   * Return the new enabled setting from the form state.
   */
  protected function getEnforced($config) {
    return $this->getConfigEnforceFromForm()[$config]['config_enforce_enabled'];
  }

  /**
   * Return the new enforcement level from the form state.
   */
  protected function getLevel($config) {
    return $this->getConfigEnforceFromForm()[$config]['config_enforce_level'];
  }

}
